@@ -102,6 +102,11 @@ class mapTiles(ranaModule):
               platform dependendt value,
               user configurable
     """
+    self.specialTiles = {
+                         'tileDownloading' : 'icons/bitmap/tile_downloading.png',
+                         'tileDownloadFailed' : 'icons/bitmap/tile_download_failed.png',
+                         'tileNetworkError' : 'icons/bitmap/tile_network_error.png'
+                        }
 
 #    self.oldZ = None
 #    self.oldThreadCount = None
@@ -194,6 +199,8 @@ class mapTiles(ranaModule):
     (also automagicaly refreshes the screen once new tiles are avalabale, even when not centered)"""
 
 #    print "nr images: %d" % len(self.images)
+#    print "nr queue: %d" % len(self.imagesQueue)
+
     
     if len(self.threads) == 0:
       return
@@ -204,6 +211,7 @@ class mapTiles(ranaModule):
       del self.threads[index]
     return
 
+
     """seems that some added error handling in the download thread class can replace this,
        but it left here for testing purposses"""
 #    z = self.get('z', 15)
@@ -227,9 +235,20 @@ class mapTiles(ranaModule):
 #        self.oldThreadCount = len(self.threads)
 #    self.oldThreadCount = len(self.threads)
 
+  def storeInMemmoryAndEnqueue(self, surface, name):
+    self.images[name] = surface
+    if name not in self.specialTiles:
+      self.imagesQueue.append(name)
+    # check cache size
+    # if there are too many images, delete them
+    if len(self.images) > self.maxImagesInMemmory:
+      self.trimCache()
+
+
+
   def trimCache(self):
     """to avoid a memmory leak, the maximum size of the image cache is fixed
-       when we reech the maximum size, we start removin images,
+       when we reech the maximum size, we start removing images,
        starting from the oldes ones
        we remove one image at a time
        """
@@ -238,6 +257,18 @@ class mapTiles(ranaModule):
     del self.images[first]
     del self.imagesQueue[0]
 #    print "images queue length:%d" % len(self.imagesQueue)
+#    print "memmory cache length:%d" % len(self.imagesQueue)
+
+  def removeNonexistingFromQueue(self):
+    # remove nonexistant images from queue
+    self.imagesQueue = filter(lambda x: x in self.images.keys(), self.imagesQueue)
+
+  def removeImageFromMemmory(self, name):
+    # remove an image from memmory
+    if name in self.images:
+      del self.images[name]
+    # make sure its removed from the queue
+    self.removeNonexistingFromQueue()
   
   def drawImage(self,cr, name, x,y):
     """Draw a tile image"""
@@ -297,7 +328,10 @@ class mapTiles(ranaModule):
     # Second, is it already in the process of being downloaded?
     if name in self.threads.keys():
       if(not self.threads[name].finished):
-        return(None)
+        dlTileName = 'tileDownloading'
+        dlTilePath = self.specialTiles[dlTileName]
+        self.loadImageFromFile(dlTilePath, dlTileName)
+        return(dlTileName)
     
     # Third, is it in the disk cache?  (including ones recently-downloaded)
     layerInfo = maplayers.get(layer, None)
@@ -334,13 +368,7 @@ class mapTiles(ranaModule):
         ct2.set_source_pixbuf(pixbuf,0,0)
         ct2.paint()
         ''' surface now contains the image in a Cairo surface '''
-        self.images[name] = surface
-        self.imagesQueue.append(name)
-
-        # check cache size
-        # if there are too many images, delete them
-        if len(self.images) > self.maxImagesInMemmory:
-          self.trimCache()
+        self.storeInMemmoryAndEnqueue(surface, name)
 
 
 
@@ -377,6 +405,26 @@ class mapTiles(ranaModule):
 #        downloadTile(x,y,z,layer,filename)
 #        return(name)
 
+  def loadImageFromFile(self,path,name):
+    pixbuf = gtk.gdk.pixbuf_new_from_file(path)
+    #x = pixbuf.get_width()
+    #y = pixbuf.get_height()
+    # Google sat images are 256 by 256 px, we dont need to check the size
+    x = 256
+    y = 256
+    ''' create a new cairo surface to place the image on '''
+    surface = cairo.ImageSurface(0,x,y)
+    ''' create a context to the new surface '''
+    ct = cairo.Context(surface)
+    ''' create a GDK formatted Cairo context to the new Cairo native context '''
+    ct2 = gtk.gdk.CairoContext(ct)
+    ''' draw from the pixbuf to the new surface '''
+    ct2.set_source_pixbuf(pixbuf,0,0)
+    ct2.paint()
+    ''' surface now contains the image in a Cairo surface '''
+    self.storeInMemmoryAndEnqueue(surface, name)
+
+
   def imageName(self,x,y,z,layer):
     """Get a unique name for a tile image 
     (suitable for use as part of filenames, dictionary keys, etc)"""
@@ -423,7 +471,43 @@ class mapTiles(ranaModule):
           self.layer,
           self.filename)
         self.finished = 1
+
+      # something is wrong with the server or url
+      except urllib2.HTTPError, e:
+        callback = self.callback
+        name = "%s_%d_%d_%d" % (self.layer,self.z,self.x,self.y)
+        errorTilePath = callback.specialTiles['tileDownloadFailed']
+        callback.loadImageFromFile(errorTilePath,name)
+        """
+        like this, when tile download fails due to a http error,
+        the error tile is loaded instead
+        like this:
+         - modRana does not immediately try to download a tile that errors out
+         - the error tile is shown without modifieng the pipeline too much
+         - modRana will eventually try to download the tile again,
+           after it is flushed with old tiles from the memmory
+        """
+      except urllib2.URLError, e:
+        callback = self.callback
+        name = "%s_%d_%d_%d" % (self.layer,self.z,self.x,self.y)
+        errorTilePath = callback.specialTiles['tileNetworkError']
+        callback.loadImageFromFile(errorTilePath,name)
+        time.sleep(10) # dont DOS the system, when we temorarily loose internet connection or other error occurs
+        # remove the image from memmory so it can be retried
+        callback.removeImageFromMemmory(name)
+        self.finished = 1 # finished thread can be removed from the set and retried
+
+
+      # something other is wrong (most probably a corrupted tile)
       except Exception, e:
+        self.printErrorMessage(e)
+
+
+
+        time.sleep(10) # dont DOS the system, when we temorarily loose internet connection or other error occurs
+        self.finished = 1 # finished thread can be removed from the set and retried
+
+    def printErrorMessage(self, e):
         url = getTileUrl(self.x,self.y,self.z,self.layer)
         print "mapTiles: download thread reports error"
         print "** we were doing this, when an exception occured:"
@@ -436,10 +520,6 @@ class mapTiles(ranaModule):
                                                                             url)
         print "** this exception occured: %s" % e
 
-
-        time.sleep(10) # dont DOS the system, when we temorarily loose internet connection or other error occurs
-        self.finished = 1 # finished thread can be removed from the set and retryed
-
     def downloadTile(self,x,y,z,layer,filename):
       """Downloads an image"""
 #      layerDetails = maplayers.get(layer, None)
@@ -453,6 +533,7 @@ class mapTiles(ranaModule):
       url = getTileUrl(x,y,z,layer)
 
       request = urllib2.urlopen(url)
+#      request = urllib.urlopen(url)
       content = request.read()
       request.close()
       name = "%s_%d_%d_%d" % (layer,z,x,y)
@@ -481,13 +562,7 @@ class mapTiles(ranaModule):
       ct2.set_source_pixbuf(pl.get_pixbuf(),0,0)
       ct2.paint()
       ''' surface now contains the image in a Cairo surface '''
-      self.callback.images[name] = surface
-      self.callback.imagesQueue.append(name)
-      
-      # check cache size
-      # if there are too many images, delete them
-      if len(self.callback.images) > self.callback.maxImagesInMemmory:
-        self.callback.trimCache()
+      self.callback.storeInMemmoryAndEnqueue(surface, name)
 
       # like this, currupted tiles should not get past the pixbuf loader and be stored
       f = open(filename, 'w') # write the tile to file
